//========================================================================================
//
//	File:				FWColor.cpp
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCOLOR_H
#include "FWColor.h"
#endif

//==============================================================================
//	 RunTime Info
//==============================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwgraphx
#endif

//==============================================================================
//	 CLASS FW_CColor
//==============================================================================

//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor()
{
#ifdef FW_BUILD_WIN
	fRGBColor = FW_kRGBBlack;
#endif
#ifdef FW_BUILD_MAC
	red = FW_kRGBBlack.red;
	green = FW_kRGBBlack.green;
	blue = FW_kRGBBlack.blue;
#endif
}

//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor(unsigned short r, unsigned short g, unsigned short b)
{
	SetRGB(r, g, b);
}

#ifdef FW_BUILD_MAC
//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor(const RGBColor* rgb)
{
	red = rgb->red;
	green = rgb->green;
	blue = rgb->blue;
}
#endif

#ifdef FW_BUILD_MAC
//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor(const RGBColor& rgb)
{
	red = rgb.red;
	green = rgb.green;
	blue = rgb.blue;
}
#endif

#ifdef FW_BUILD_WIN
//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor(COLORREF rgb) :
	fRGBColor(rgb)
{
}
#endif

//------------------------------------------------------------------------------
//	 FW_CColor::FW_CColor
//------------------------------------------------------------------------------

FW_CColor::FW_CColor(const FW_CColor& color)
{
#ifdef FW_BUILD_WIN
	fRGBColor = color.fRGBColor;
#endif
#ifdef FW_BUILD_MAC
	red = color.red;
	green = color.green;
	blue = color.blue;
#endif
}

#ifdef FW_BUILD_WIN
//------------------------------------------------------------------------------
//	 FW_CColor::operator=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator=(COLORREF rgb)
{
	fRGBColor = rgb;
	return (*this);
}
#endif

//------------------------------------------------------------------------------
//	 FW_CColor::operator=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator=(const FW_CColor& color)
{
#ifdef FW_BUILD_WIN
	fRGBColor = color.fRGBColor;
#endif
#ifdef FW_BUILD_MAC
	red = color.red;
	green = color.green;
	blue = color.blue;
#endif
	return (*this);
}

//------------------------------------------------------------------------------
// Arithmatic, by another color for FW_CColor
// Only + and - really make sense for FW_CColor operands, but multiply and divide
// exist for consistency, since they do make sense for scalars
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	FW_CColor::operator+
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator+(const FW_CColor& color) const
{
	FW_CColor newColor(*this);
	return newColor += color;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator-
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator -(const FW_CColor& color) const
{
	FW_CColor newColor(*this);
	return newColor -= color;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator*
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator *(const FW_CColor& color) const
{
	FW_CColor newColor(*this);
	return newColor *= color;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator/
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator /(const FW_CColor& color) const
{
	FW_CColor newColor(*this);
	return newColor /= color;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator +=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator+=(const FW_CColor& color)
{
	short red1, green1, blue1, red2, green2, blue2;

	GetRGB(red1, green1, blue1);
	color.GetRGB(red2, green2, blue2);
	
	AddComponents(red1, red2);
	AddComponents(green1, green2);
	AddComponents(blue1, blue2);

	SetRGB(red1, green1, blue1);

	return *this;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator -=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator-=(const FW_CColor& color)
{
	short red1, green1, blue1, red2, green2, blue2;

	GetRGB(red1, green1, blue1);
	color.GetRGB(red2, green2, blue2);
	
	SetRGB(SubtractComponents(red1, red2), 
			SubtractComponents(green1, green2),
			SubtractComponents(blue1, blue2));

	return *this;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator *=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator*=(const FW_CColor& color)
{
	short red1, green1, blue1, red2, green2, blue2;

	GetRGB(red1, green1, blue1);
	color.GetRGB(red2, green2, blue2);
	
	SetRGB(MultiplyComponents(red1, red2), 
			MultiplyComponents(green1, green2), 
			MultiplyComponents(blue1, blue2));

	return *this;
}

//------------------------------------------------------------------------------
//	FW_CColor::operator /=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator/=(const FW_CColor& color)
{
	short red1, green1, blue1, red2, green2, blue2;

	GetRGB(red1, green1, blue1);
	color.GetRGB(red2, green2, blue2);
	
	SetRGB(DivideComponents(red1, red2), 
			DivideComponents(green1, green2), 
			DivideComponents(blue1, blue2));

	return *this;
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator +
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator+(unsigned short aScalar) const
{
	FW_CColor newColor(*this);

	return (newColor += aScalar);
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator -
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator-(unsigned short aScalar) const
{
	FW_CColor newColor(*this);

	return (newColor -= aScalar);
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator *
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator*(unsigned short aScalar) const
{
	FW_CColor newColor(*this);

	return (newColor *= aScalar);
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator /
//------------------------------------------------------------------------------

FW_CColor FW_CColor::operator/(unsigned short aScalar) const
{
	FW_CColor newColor(*this);

	return (newColor /= aScalar);
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator +=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator+=(unsigned short aScalar)
{
	short red, green, blue;

	GetRGB(red, green, blue);
	
	SetRGB(AddComponents(red, aScalar), 
			AddComponents(green, aScalar), 
			AddComponents(blue, aScalar));

	return *this;
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator -=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator-=(unsigned short aScalar)
{
	short red, green, blue;

	GetRGB(red, green, blue);
	
	SetRGB(SubtractComponents(red, aScalar), 
			SubtractComponents(green, aScalar), 
			SubtractComponents(blue, aScalar));

	return *this;
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator *=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator*=(unsigned short aScalar)
{
	short red, green, blue;

	GetRGB(red, green, blue);
	
	SetRGB(MultiplyComponents(red, aScalar), 
			MultiplyComponents(green, aScalar), 
			MultiplyComponents(blue, aScalar));

	return *this;
}

//------------------------------------------------------------------------------
//	 FW_CColor::operator /=
//------------------------------------------------------------------------------

FW_CColor& FW_CColor::operator/=(unsigned short aScalar)
{
	short red, green, blue;

	GetRGB(red, green, blue);
	
	SetRGB(DivideComponents(red, aScalar), 
		   DivideComponents(green, aScalar), 
		   DivideComponents(blue, aScalar));

	return *this;
}

//------------------------------------------------------------------------------
//	 FW_CColor::Grayer
//------------------------------------------------------------------------------

void FW_CColor::Grayer()
{
	short red, green, blue;

	GetRGB(red, green, blue);
	
	SetRGB(0x003F + (red >> 1), 0x003F + (green >> 1), 0x003F + (blue >> 1));
}

//------------------------------------------------------------------------------
//	 FW_CColor::Brightness
//------------------------------------------------------------------------------

unsigned long FW_CColor::Brightness() const
{
	short red, green, blue;

	GetRGB(red, green, blue);

	// calculate approximate brightness using YIQ formula where
	// Y = lumenance = .30R + .59G + .11B
	// approx. fY := (r / 4) + 5 * (g / 8) + b / 8  (.25 R + .625 G + .125 B)

	return (red >> 2) + (5L * (green >> 3)) + (blue >> 3);
}

//------------------------------------------------------------------------------
//	 FW_CColor::IsDarkerThan
//------------------------------------------------------------------------------

FW_Boolean FW_CColor::IsDarkerThan(const FW_CColor& color) const
{
	return Brightness() < color.Brightness();
}

//------------------------------------------------------------------------------
//	 FW_CColor::IsLighterThan
//------------------------------------------------------------------------------

FW_Boolean FW_CColor::IsLighterThan(const FW_CColor& color) const
{
	return Brightness() > color.Brightness();
}

//------------------------------------------------------------------------------
//	 FW_CColor::SetRGB
//------------------------------------------------------------------------------

void FW_CColor::SetRGB(unsigned short r, unsigned short g, unsigned short b)
{
#ifdef FW_BUILD_MAC
	red = r | (r << 8);
	green = g | (g << 8);
	blue = b | (b << 8);
#endif
#ifdef FW_BUILD_WIN
	fRGBColor = RGB(r, g, b);
#endif
}

//------------------------------------------------------------------------------
//	 FW_CColor::GetRGB
//------------------------------------------------------------------------------

void FW_CColor::GetRGB(unsigned short& r, unsigned short& g, unsigned short& b) const
{
#ifdef FW_BUILD_WIN
	r = (unsigned short)(fRGBColor & 0x000000FF);
	g = (unsigned short)((fRGBColor & 0x0000FF00) >> 8);
	b = (unsigned short)((fRGBColor & 0x00FF0000) >> 16);
#endif

#ifdef FW_BUILD_MAC
	r = (red & 0xFF00) >> 8;
	g = (green & 0xFF00) >> 8;
	b = (blue & 0xFF00) >> 8;
#endif
}

//------------------------------------------------------------------------------
//	 FW_CColor::Unflatten
//------------------------------------------------------------------------------
void FW_CColor::Unflatten(FW_CReadableStream& stream)
{
	unsigned short rgb[3];
	stream.Read(rgb, 3);
	SetRGB(rgb[0], rgb[1], rgb[2]);
}

//------------------------------------------------------------------------------
//	 FW_CColor::Flatten
//------------------------------------------------------------------------------
void FW_CColor::Flatten(FW_CWritableStream& stream)
{
	unsigned short rgb[3];
	GetRGB(rgb[0], rgb[1], rgb[2]);	
	stream.Write(rgb, 3);
}


//------------------------------------------------------------------------------
//	 FW_CColor::AddComponents
//------------------------------------------------------------------------------

unsigned short FW_CColor::AddComponents(unsigned short a, unsigned short b)
{
	a += b;

	if (a > 255)
		a = 255;
	
	return a;
}

//------------------------------------------------------------------------------
//	 FW_CColor::SubtractComponents
//------------------------------------------------------------------------------

unsigned short FW_CColor::SubtractComponents(unsigned short a, unsigned short b)
{
	return a <= b ? 0 : a - b;
}

//------------------------------------------------------------------------------
//	 FW_CColor::MultiplyComponents
//------------------------------------------------------------------------------

unsigned short FW_CColor::MultiplyComponents(unsigned short a, unsigned short b)
{
	a *= b;

	if (a > 255)
		a = 255;

	return a;
}

//------------------------------------------------------------------------------
//	 FW_CColor::DivideComponents
//------------------------------------------------------------------------------

unsigned short FW_CColor::DivideComponents(unsigned short a, unsigned short b)
{
	return b == 0 ? 255 : a / b;
}


